#!/bin/sh -x

##############################################################################
#
# Arguments:
# $1 - the "from" version
# $2 - the "to" version
# $3 - the root directory (that is, the directory to treat as if it were / )
#
# This controls the kernel and fpga flashing process. We expect that the 
# fpga and kernel files will be delivered to the disk by the normal upgrade
# process. We then look to see of the version numbers have changed, and if 
# they have, we go ahead and flash the kernel.
#
# We default the versions to the empty string. When we upgrade the kernel, we
# need to change these versions. Note that machines with upgraded kernels or
# fpgas should ship with the files /etc/solartech/kernelversion and 
# /etc/solartech/fpgaversion, containing the current versions.

# If this is -s, we will go through the motions of flashing, without actually
# doing the flashing itself.
FLASH_SIMULATE=

. /etc/flash_versions.sh

# Sync
echo Syncing to free up memory...
/bin/sync
sleep 5
/bin/sync
sleep 5

echo Beginning flash...

# Backup FPGA.
echo 'Backup FPGA... (on disk: ' `cat /etc/solartech/backup_fpgaversion` ', will be: ' $BACKUP_FVERSION ')'
/bin/sync

if [ -e $BACKUP_FPGA -a ! x`cat /etc/solartech/backup_fpgaversion` = x$BACKUP_FVERSION ]; then
    zcat /usr/update_manager/screens/flashing_fpga.screen.gz | tofd $UPGRADEGUI
    if /usr/bin/upgrade_flash_new $FLASH_SIMULATE -g $BACKUP_FPGA -p $UPGRADEGUI ; then
        echo $BACKUP_FVERSION > /etc/solartech/backup_fpgaversion
    else
        zcat /usr/update_manager/screens/error.screen.gz | tofd $UPGRADEGUI
        echo Q | tofd $UPGRADEGUI
        /usr/bin/report_status_on_sign FAILURE

        if test -e /upgrade.stu; then
            sleep 300
            /usr/bin/rtc_reboot
        fi
        
        sleep 86400
        exit -1

    fi
else
    echo Backup FPGA flash not needed.
fi

echo Done with backup FPGA.
/bin/sync

# Backup Kernel
echo 'Backup Kernel... (on disk: ' `cat /etc/solartech/backup_kernelversion` ', will be: ' $BACKUP_KVERSION ')'
if [ -e $BACKUP_KERNEL -a ! x`cat /etc/solartech/backup_kernelversion` = x$BACKUP_KVERSION ]; then
    zcat /usr/update_manager/screens/flashing_kernel.screen.gz | tofd $UPGRADEGUI
    if /usr/bin/upgrade_flash_new $FLASH_SIMULATE -l $BACKUP_KERNEL -p $UPGRADEGUI; then
        echo $BACKUP_KVERSION > /etc/solartech/backup_kernelversion
    else
        zcat /usr/update_manager/screens/error.screen.gz | tofd $UPGRADEGUI
        echo Q | tofd $UPGRADEGUI
        /usr/bin/report_status_on_sign FAILURE

        if test -e /upgrade.stu; then
            sleep 300
            /usr/bin/rtc_reboot
        fi
        
        sleep 86400
        exit -1

    fi
else
    echo Backup kernel flash not needed.
fi

echo Done with backup kernel.
/bin/sync


# Tertiary and Primary FPGA
echo 'Primary FPGA... (on disk: ' `cat /etc/solartech/fpgaversion` ', will be: ' $FLASH_FVERSION ')'
if [ -e $FLASH_FPGA -a ! x`cat /etc/solartech/fpgaversion` = x$FLASH_FVERSION ]; then
    zcat /usr/update_manager/screens/flashing_fpga.screen.gz | tofd $UPGRADEGUI
    
    # Backup.
    if /usr/bin/upgrade_flash_new $FLASH_SIMULATE -h $FLASH_FPGA -p $UPGRADEGUI; then
        # Dont bother storing success until we flash the primary.
        true
    else
        zcat /usr/update_manager/screens/error.screen.gz | tofd $UPGRADEGUI
        echo Q | tofd $UPGRADEGUI
        /usr/bin/report_status_on_sign FAILURE

        if test -e /upgrade.stu; then
            sleep 300
            /usr/bin/rtc_reboot
        fi
        
        sleep 86400
        exit -1

    fi

    zcat /usr/update_manager/screens/flashing_fpga.screen.gz | tofd $UPGRADEGUI
    
    if /usr/bin/upgrade_flash_new $FLASH_SIMULATE -f $FLASH_FPGA -p $UPGRADEGUI; then
        echo $FLASH_FVERSION > /etc/solartech/fpgaversion
    else
        zcat /usr/update_manager/screens/error.screen.gz | tofd $UPGRADEGUI
        echo Q | tofd $UPGRADEGUI
        /usr/bin/report_status_on_sign FAILURE

        if test -e /upgrade.stu; then
            sleep 300
            /usr/bin/rtc_reboot
        fi
        
        sleep 86400
        exit -1

    fi

else
    echo FPGA flash not needed.
fi

echo Done with primary FPGA.
/bin/sync

# Primary Kernel
echo 'Primary Kernel... (on disk: ' `cat /etc/solartech/kernelversion` ', will be: ' $FLASH_KVERSION ')'
if [ -e $FLASH_KERNEL -a ! x`cat /etc/solartech/kernelversion` = x$FLASH_KVERSION ]; then
    zcat /usr/update_manager/screens/flashing_kernel.screen.gz | tofd $UPGRADEGUI

    echo kernel name is $FLASH_KERNEL

    echo /usr/bin/upgrade_flash_new $FLASH_SIMULATE -k $FLASH_KERNEL -p $UPGRADEGUI
    
    if /usr/bin/upgrade_flash_new $FLASH_SIMULATE -k $FLASH_KERNEL -p $UPGRADEGUI; then
        echo $FLASH_KVERSION > /etc/solartech/kernelversion
    else
        zcat /usr/update_manager/screens/error.screen.gz | tofd $UPGRADEGUI
        echo Q | tofd $UPGRADEGUI
        /usr/bin/report_status_on_sign FAILURE
        if test -e /upgrade.stu; then
            sleep 300
            /usr/bin/rtc_reboot
        fi
        
        sleep 86400
        exit -1
    fi
else
    echo Kernel flash not needed.
fi

echo Done with primary kernel.
/bin/sync

# Run the post-upgrade script
if [ -x /etc/post_upgrade.sh ]; then
    /bin/sh /etc/post_upgrade.sh
fi

# Record the version number.
echo -n "$2" > "$3etc/solartech/version"

# Sync the disk a few times.
sync
sync

# Report success.
zcat "$3usr/update_manager/screens/success.screen.gz" | tofd $UPGRADEGUI
echo -n Q | tofd $UPGRADEGUI
/usr/bin/report_status_on_sign SUCCESS &

rm /upgrade.go
rm /upgrade.repeat

if [ -e /etc/solartech/first_time ]; then
    rm /etc/solartech/first_time
    sync
fi

# exercise most of our peripherals in a desparate attempt to have things come up working
/usr/bin/devices_exerciser

# This is responsible for terminating the system, as necessary.
if test -e /upgrade.stu; then
    sleep 30
    # this code can be removed when we stop supporting TRAFIX-2.0.8 and Agility 2.0.6
    for micro in 0 1 2 3 4 5 6 7 8; do
        if [ "$1" = "TRAFIX-2.0.$micro" ]
        then
            /usr/bin/legacy_rtc_reboot
        fi
    done
    for micro in 0 1 2 3 4 5 6; do
        if [ "$1" = "Agility-2.0.$micro" ]
        then
            /usr/bin/legacy_rtc_reboot
        fi
    done
    # end of removable code

    /usr/bin/rtc_reboot
fi

while [ 1 == 1 ]; do sleep 86400 ; done
exit -1
